@@ -16,11 +16,14 @@ from traitlets import CaselessStrEnum
 from traitlets import Dict
 from traitlets import HasTraits
 from traitlets import List
+from traitlets import observe
 from traitlets import Set
 from traitlets import Type
 from traitlets import Unicode
 from traitlets.config import LoggingConfigurable  # type: ignore
 
+from .provisioning import KernelProvisionerFactory as KPF
+
 pjoin = os.path.join
 
 NATIVE_KERNEL_NAME = "python3"
@@ -28,6 +31,8 @@ NATIVE_KERNEL_NAME = "python3"
 
 class KernelSpec(HasTraits):
     argv = List()
+    name = Unicode()
+    mimetype = Unicode()
     display_name = Unicode()
     language = Unicode()
     env = Dict()
@@ -146,7 +151,12 @@ class KernelSpecManager(LoggingConfigurable):
 
     whitelist = Set(
         config=True,
-        help="""Whitelist of allowed kernel names.
+        help="""Deprecated, use `KernelSpecManager.allowed_kernelspecs`
+        """,
+    )
+    allowed_kernelspecs = Set(
+        config=True,
+        help="""List of allowed kernel names.
 
         By default, all installed kernels are allowed.
         """,
@@ -155,6 +165,35 @@ class KernelSpecManager(LoggingConfigurable):
         help="List of kernel directories to search. Later ones take priority over earlier."
     )
 
+    _deprecated_aliases = {
+        "whitelist": ("allowed_kernelspecs", "7.0"),
+    }
+
+    # Method copied from
+    # https://github.com/jupyterhub/jupyterhub/blob/d1a85e53dccfc7b1dd81b0c1985d158cc6b61820/jupyterhub/auth.py#L143-L161
+    @observe(*list(_deprecated_aliases))
+    def _deprecated_trait(self, change):
+        """observer for deprecated traits"""
+        old_attr = change.name
+        new_attr, version = self._deprecated_aliases.get(old_attr)
+        new_value = getattr(self, new_attr)
+        if new_value != change.new:
+            # only warn if different
+            # protects backward-compatible config from warnings
+            # if they set the same value under both names
+            self.log.warning(
+                (
+                    "{cls}.{old} is deprecated in jupyter_client "
+                    "{version}, use {cls}.{new} instead"
+                ).format(
+                    cls=self.__class__.__name__,
+                    old=old_attr,
+                    new=new_attr,
+                    version=version,
+                )
+            )
+            setattr(self, new_attr, change.new)
+
     def _kernel_dirs_default(self):
         dirs = jupyter_path("kernels")
         # At some point, we should stop adding .ipython/kernels to the path,
@@ -194,9 +233,9 @@ class KernelSpecManager(LoggingConfigurable):
             except ImportError:
                 self.log.warning("Native kernel (%s) is not available", NATIVE_KERNEL_NAME)
 
-        if self.whitelist:
-            # filter if there's a whitelist
-            d = {name: spec for name, spec in d.items() if name in self.whitelist}
+        if self.allowed_kernelspecs:
+            # filter if there's an allow list
+            d = {name: spec for name, spec in d.items() if name in self.allowed_kernelspecs}
         return d
         # TODO: Caching?
 
@@ -204,6 +243,7 @@ class KernelSpecManager(LoggingConfigurable):
         """Returns a :class:`KernelSpec` instance for a given kernel_name
         and resource_dir.
         """
+        kspec = None
         if kernel_name == NATIVE_KERNEL_NAME:
             try:
                 from ipykernel.kernelspec import RESOURCES, get_kernel_dict
@@ -212,9 +252,14 @@ class KernelSpecManager(LoggingConfigurable):
                 pass
             else:
                 if resource_dir == RESOURCES:
-                    return self.kernel_spec_class(resource_dir=resource_dir, **get_kernel_dict())
+                    kspec = self.kernel_spec_class(resource_dir=resource_dir, **get_kernel_dict())
+        if not kspec:
+            kspec = self.kernel_spec_class.from_resource_dir(resource_dir)
 
-        return self.kernel_spec_class.from_resource_dir(resource_dir)
+        if not KPF.instance(parent=self.parent).is_provisioner_available(kspec):
+            raise NoSuchKernel(kernel_name)
+
+        return kspec
 
     def _find_spec_directory(self, kernel_name):
         """Find the resource directory of a named kernel spec"""
@@ -240,13 +285,12 @@ class KernelSpecManager(LoggingConfigurable):
         """
         if not _is_valid_kernel_name(kernel_name):
             self.log.warning(
-                "Kernelspec name %r is invalid: %s",
-                kernel_name,
-                _kernel_name_description,
+                f"Kernelspec name {kernel_name} is invalid: {_kernel_name_description}"
             )
 
         resource_dir = self._find_spec_directory(kernel_name.lower())
         if resource_dir is None:
+            self.log.warning(f"Kernelspec name {kernel_name} cannot be found!")
             raise NoSuchKernel(kernel_name)
 
         return self._get_kernel_spec_by_name(kernel_name, resource_dir)
@@ -277,6 +321,8 @@ class KernelSpecManager(LoggingConfigurable):
                     spec = self.get_kernel_spec(kname)
 
                 res[kname] = {"resource_dir": resource_dir, "spec": spec.to_dict()}
+            except NoSuchKernel:
+                pass  # The appropriate warning has already been logged
             except Exception:
                 self.log.warning("Error loading kernelspec %r", kname, exc_info=True)
         return res
