@@ -17,6 +17,7 @@ from traitlets.config.application import Application
 
 from . import __version__
 from .kernelspec import KernelSpecManager
+from .provisioning.factory import KernelProvisionerFactory
 
 
 class ListKernelSpecs(JupyterApp):
@@ -181,7 +182,7 @@ class RemoveKernelSpec(JupyterApp):
         return KernelSpecManager(data_dir=self.data_dir, parent=self)
 
     flags = {
-        "f": ({"RemoveKernelSpec": {"force": True}}, force.get_metadata("help")),
+        "f": ({"RemoveKernelSpec": {"force": True}}, force.help),
     }
     flags.update(JupyterApp.flags)
 
@@ -200,7 +201,7 @@ class RemoveKernelSpec(JupyterApp):
         if missing:
             self.exit("Couldn't find kernel spec(s): %s" % ", ".join(missing))
 
-        if not self.force:
+        if not (self.force or self.answer_yes):
             print("Kernel specs to remove:")
             for name in self.spec_names:
                 print("  %s\t%s" % (name.ljust(20), spec_paths[name]))
@@ -270,6 +271,22 @@ class InstallNativeKernelSpec(JupyterApp):
             self.exit(e)
 
 
+class ListProvisioners(JupyterApp):
+    version = __version__
+    description = """List available provisioners for use in kernel specifications."""
+
+    def start(self):
+        kfp = KernelProvisionerFactory.instance(parent=self)
+        print("Available kernel provisioners:")
+        provisioners = kfp.get_provisioner_entries()
+
+        # pad to width of longest kernel name
+        name_len = len(sorted(provisioners, key=lambda name: len(name))[-1])
+
+        for name in sorted(provisioners):
+            print(f"  {name.ljust(name_len)}    {provisioners[name]}")
+
+
 class KernelSpecApp(Application):
     version = __version__
     name = "jupyter kernelspec"
@@ -288,6 +305,7 @@ class KernelSpecApp(Application):
                 InstallNativeKernelSpec,
                 InstallNativeKernelSpec.description.splitlines()[0],
             ),
+            "provisioners": (ListProvisioners, ListProvisioners.description.splitlines()[0]),
         }
     )
 
