@@ -12,6 +12,7 @@ Sessions.
 # Distributed under the terms of the Modified BSD License.
 import hashlib
 import hmac
+import json
 import logging
 import os
 import pickle
@@ -45,11 +46,11 @@ from traitlets.log import get_logger  # type: ignore
 from traitlets.utils.importstring import import_item  # type: ignore
 from zmq.eventloop.ioloop import IOLoop
 from zmq.eventloop.zmqstream import ZMQStream
-from zmq.utils import jsonapi
 
 from jupyter_client import protocol_version
 from jupyter_client.adapter import adapt
 from jupyter_client.jsonutil import extract_dates
+from jupyter_client.jsonutil import json_clean
 from jupyter_client.jsonutil import json_default
 from jupyter_client.jsonutil import squash_dates
 
@@ -92,16 +93,36 @@ MAX_BYTES = 1024
 
 
 def json_packer(obj):
-    return jsonapi.dumps(
-        obj,
-        default=json_default,
-        ensure_ascii=False,
-        allow_nan=False,
-    )
+    try:
+        return json.dumps(
+            obj,
+            default=json_default,
+            ensure_ascii=False,
+            allow_nan=False,
+        ).encode("utf8")
+    except (TypeError, ValueError) as e:
+        # Fallback to trying to clean the json before serializing
+        packed = json.dumps(
+            json_clean(obj),
+            default=json_default,
+            ensure_ascii=False,
+            allow_nan=False,
+        ).encode("utf8")
+
+        warnings.warn(
+            f"Message serialization failed with:\n{e}\n"
+            "Supporting this message is deprecated in jupyter-client 7, please make "
+            "sure your message is JSON-compliant",
+            stacklevel=2,
+        )
+
+        return packed
 
 
 def json_unpacker(s):
-    return jsonapi.loads(s)
+    if isinstance(s, bytes):
+        s = s.decode("utf8", "replace")
+    return json.loads(s)
 
 
 def pickle_packer(o):
@@ -444,7 +465,7 @@ class Session(Configurable):
 
     digest_history = Set()
     digest_history_size = Integer(
-        2 ** 16,
+        2**16,
         config=True,
         help="""The maximum number of digests to remember.
 
@@ -483,7 +504,7 @@ class Session(Configurable):
 
     # thresholds:
     copy_threshold = Integer(
-        2 ** 16,
+        2**16,
         config=True,
         help="Threshold (in bytes) beyond which a buffer should be sent without copying.",
     )
@@ -589,12 +610,9 @@ class Session(Configurable):
         try:
             packed = pack(msg_list)
         except Exception as e:
-            error_msg = "packer '{packer}' could not serialize a simple message: {e}{jsonmsg}"
-            if self.packer == "json":
-                jsonmsg = "\nzmq.utils.jsonapi.jsonmod = %s" % jsonapi.jsonmod
-            else:
-                jsonmsg = ""
-            raise ValueError(error_msg.format(packer=self.packer, e=e, jsonmsg=jsonmsg)) from e
+            raise ValueError(
+                f"packer '{self.packer}' could not serialize a simple message: {e}"
+            ) from e
 
         # ensure packed message is bytes
         if not isinstance(packed, bytes):
@@ -605,15 +623,9 @@ class Session(Configurable):
             unpacked = unpack(packed)
             assert unpacked == msg_list
         except Exception as e:
-            error_msg = (
-                "unpacker '{unpacker}' could not handle output from packer '{packer}': {e}{jsonmsg}"
-            )
-            if self.packer == "json":
-                jsonmsg = "\nzmq.utils.jsonapi.jsonmod = %s" % jsonapi.jsonmod
-            else:
-                jsonmsg = ""
             raise ValueError(
-                error_msg.format(packer=self.packer, unpacker=self.unpacker, e=e, jsonmsg=jsonmsg)
+                f"unpacker '{self.unpacker}' could not handle output from packer"
+                f" '{self.packer}': {e}"
             ) from e
 
         # check datetime support
