@@ -19,7 +19,7 @@ from multiprocessing import Process
 try:
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", DeprecationWarning)
-        import paramiko
+        import paramiko  # type: ignore
 
         SSHException = paramiko.ssh_exception.SSHException
 except ImportError:
@@ -28,7 +28,6 @@ except ImportError:
     class SSHException(Exception):  # type: ignore
         pass
 
-
 else:
     from .forward import forward_tunnel
 
@@ -37,8 +36,6 @@ try:
 except ImportError:
     pexpect = None
 
-from zmq.utils.strtypes import b
-
 
 def select_random_ports(n):
     """Select and return n random ports that are available."""
@@ -57,7 +54,7 @@ def select_random_ports(n):
 # -----------------------------------------------------------------------------
 # Check for passwordless login
 # -----------------------------------------------------------------------------
-_password_pat = re.compile(b(r"pass(word|phrase):"), re.IGNORECASE)
+_password_pat = re.compile((r"pass(word|phrase):".encode("utf8")), re.IGNORECASE)
 
 
 def try_passwordless_ssh(server, keyfile, paramiko=None):
