@@ -11,14 +11,19 @@ import os
 def run_sync(coro):
     def wrapped(*args, **kwargs):
         try:
-            loop = asyncio.get_event_loop()
+            loop = asyncio.get_running_loop()
         except RuntimeError:
             loop = asyncio.new_event_loop()
             asyncio.set_event_loop(loop)
         import nest_asyncio  # type: ignore
 
         nest_asyncio.apply(loop)
-        return loop.run_until_complete(coro(*args, **kwargs))
+        future = asyncio.ensure_future(coro(*args, **kwargs), loop=loop)
+        try:
+            return loop.run_until_complete(future)
+        except BaseException as e:
+            future.cancel()
+            raise e
 
     wrapped.__doc__ = coro.__doc__
     return wrapped
